@@ -1,14 +1,14 @@
 //! Operators in ErgoTree
 
-use ergotree_ir::mir::bin_op::ArithOp;
-use ergotree_ir::mir::bin_op::BinOp;
+use ergotree_ir::bigint256::BigInt256;
 use ergotree_ir::mir::bin_op::BinOpKind;
 use ergotree_ir::mir::bin_op::RelationOp;
+use ergotree_ir::mir::bin_op::{ArithOp, LogicalOp};
+use ergotree_ir::mir::bin_op::{BinOp, BitOp};
 use ergotree_ir::mir::constant::TryExtractFrom;
 use ergotree_ir::mir::constant::TryExtractInto;
 use ergotree_ir::mir::value::Value;
 use eval::costs::Costs;
-use num_bigint::BigInt;
 use num_traits::CheckedAdd;
 use num_traits::CheckedDiv;
 use num_traits::CheckedMul;
@@ -16,7 +16,6 @@ use num_traits::CheckedSub;
 use num_traits::Num;
 
 use crate::eval;
-use crate::eval::bigint::fits_in_256_bits;
 use crate::eval::env::Env;
 use crate::eval::EvalContext;
 use crate::eval::EvalError;
@@ -34,17 +33,6 @@ fn arithmetic_err<T: std::fmt::Display>(
     ))
 }
 
-fn check_bigint_overflow(val: Value) -> Result<Value, EvalError> {
-    let bigint = val.try_extract_into::<BigInt>()?;
-    if fits_in_256_bits(&bigint) {
-        Ok(bigint.into())
-    } else {
-        Err(EvalError::ArithmeticException(
-            "Arithmetic Overflow on BigInt operation".to_string(),
-        ))
-    }
-}
-
 fn eval_plus<T>(lv_raw: T, rv: Value) -> Result<Value, EvalError>
 where
     T: Num + CheckedAdd + TryExtractFrom<Value> + Into<Value> + std::fmt::Display,
@@ -104,7 +92,7 @@ fn eval_ge(lv: Value, rv: Value) -> Result<Value, EvalError> {
         Value::Short(lv_raw) => Ok((lv_raw >= rv.try_extract_into::<i16>()?).into()),
         Value::Int(lv_raw) => Ok((lv_raw >= rv.try_extract_into::<i32>()?).into()),
         Value::Long(lv_raw) => Ok((lv_raw >= rv.try_extract_into::<i64>()?).into()),
-        Value::BigInt(lv_raw) => Ok((lv_raw >= rv.try_extract_into::<BigInt>()?).into()),
+        Value::BigInt(lv_raw) => Ok((lv_raw >= rv.try_extract_into::<BigInt256>()?).into()),
         _ => Err(EvalError::UnexpectedValue(format!(
             "expected BinOp::left to be numeric value, got {0:?}",
             lv
@@ -118,7 +106,7 @@ fn eval_gt(lv: Value, rv: Value) -> Result<Value, EvalError> {
         Value::Short(lv_raw) => Ok((lv_raw > rv.try_extract_into::<i16>()?).into()),
         Value::Int(lv_raw) => Ok((lv_raw > rv.try_extract_into::<i32>()?).into()),
         Value::Long(lv_raw) => Ok((lv_raw > rv.try_extract_into::<i64>()?).into()),
-        Value::BigInt(lv_raw) => Ok((lv_raw > rv.try_extract_into::<BigInt>()?).into()),
+        Value::BigInt(lv_raw) => Ok((lv_raw > rv.try_extract_into::<BigInt256>()?).into()),
         _ => Err(EvalError::UnexpectedValue(format!(
             "expected BinOp::left to be numeric value, got {0:?}",
             lv
@@ -132,7 +120,7 @@ fn eval_lt(lv: Value, rv: Value) -> Result<Value, EvalError> {
         Value::Short(lv_raw) => Ok((lv_raw < rv.try_extract_into::<i16>()?).into()),
         Value::Int(lv_raw) => Ok((lv_raw < rv.try_extract_into::<i32>()?).into()),
         Value::Long(lv_raw) => Ok((lv_raw < rv.try_extract_into::<i64>()?).into()),
-        Value::BigInt(lv_raw) => Ok((lv_raw < rv.try_extract_into::<BigInt>()?).into()),
+        Value::BigInt(lv_raw) => Ok((lv_raw < rv.try_extract_into::<BigInt256>()?).into()),
         _ => Err(EvalError::UnexpectedValue(format!(
             "expected BinOp::left to be numeric value, got {0:?}",
             lv
@@ -146,7 +134,7 @@ fn eval_le(lv: Value, rv: Value) -> Result<Value, EvalError> {
         Value::Short(lv_raw) => Ok((lv_raw <= rv.try_extract_into::<i16>()?).into()),
         Value::Int(lv_raw) => Ok((lv_raw <= rv.try_extract_into::<i32>()?).into()),
         Value::Long(lv_raw) => Ok((lv_raw <= rv.try_extract_into::<i64>()?).into()),
-        Value::BigInt(lv_raw) => Ok((lv_raw <= rv.try_extract_into::<BigInt>()?).into()),
+        Value::BigInt(lv_raw) => Ok((lv_raw <= rv.try_extract_into::<BigInt256>()?).into()),
         _ => Err(EvalError::UnexpectedValue(format!(
             "expected BinOp::left to be numeric value, got {0:?}",
             lv
@@ -174,9 +162,24 @@ impl Evaluable for BinOp {
     fn eval(&self, env: &Env, ctx: &mut EvalContext) -> Result<Value, EvalError> {
         ctx.cost_accum.add(Costs::DEFAULT.eq_const_size)?;
         let lv = self.left.eval(env, ctx)?;
-        // using closure to keep right value from evaluation (for lazy AND, OR)
+        // using closure to keep right value from evaluation (for lazy AND, OR, XOR)
         let mut rv = || self.right.eval(env, ctx);
         match self.kind {
+            BinOpKind::Logical(op) => match op {
+                LogicalOp::And => Ok(Value::Boolean(if lv.try_extract_into::<bool>()? {
+                    rv()?.try_extract_into::<bool>()?
+                } else {
+                    false
+                })),
+                LogicalOp::Or => Ok(Value::Boolean(if !lv.try_extract_into::<bool>()? {
+                    rv()?.try_extract_into::<bool>()?
+                } else {
+                    true
+                })),
+                LogicalOp::Xor => Ok(Value::Boolean(
+                    lv.try_extract_into::<bool>()? ^ rv()?.try_extract_into::<bool>()?,
+                )),
+            },
             BinOpKind::Relation(op) => match op {
                 RelationOp::Eq => Ok(Value::Boolean(lv == rv()?)),
                 RelationOp::NEq => Ok(Value::Boolean(lv != rv()?)),
@@ -184,16 +187,6 @@ impl Evaluable for BinOp {
                 RelationOp::Lt => eval_lt(lv, rv()?),
                 RelationOp::Ge => eval_ge(lv, rv()?),
                 RelationOp::Le => eval_le(lv, rv()?),
-                RelationOp::And => Ok(Value::Boolean(if lv.try_extract_into::<bool>()? {
-                    rv()?.try_extract_into::<bool>()?
-                } else {
-                    false
-                })),
-                RelationOp::Or => Ok(Value::Boolean(if !lv.try_extract_into::<bool>()? {
-                    rv()?.try_extract_into::<bool>()?
-                } else {
-                    true
-                })),
             },
             BinOpKind::Arith(op) => match op {
                 ArithOp::Plus => match lv {
@@ -201,7 +194,7 @@ impl Evaluable for BinOp {
                     Value::Short(lv_raw) => eval_plus(lv_raw, rv()?),
                     Value::Int(lv_raw) => eval_plus(lv_raw, rv()?),
                     Value::Long(lv_raw) => eval_plus(lv_raw, rv()?),
-                    Value::BigInt(lv_raw) => check_bigint_overflow(eval_plus(lv_raw, rv()?)?),
+                    Value::BigInt(lv_raw) => eval_plus(lv_raw, rv()?),
                     _ => Err(EvalError::UnexpectedValue(format!(
                         "expected BinOp::left to be numeric value, got {0:?}",
                         lv
@@ -212,7 +205,7 @@ impl Evaluable for BinOp {
                     Value::Short(lv_raw) => eval_minus(lv_raw, rv()?),
                     Value::Int(lv_raw) => eval_minus(lv_raw, rv()?),
                     Value::Long(lv_raw) => eval_minus(lv_raw, rv()?),
-                    Value::BigInt(lv_raw) => check_bigint_overflow(eval_minus(lv_raw, rv()?)?),
+                    Value::BigInt(lv_raw) => eval_minus(lv_raw, rv()?),
                     _ => Err(EvalError::UnexpectedValue(format!(
                         "expected BinOp::left to be numeric value, got {0:?}",
                         lv
@@ -223,7 +216,7 @@ impl Evaluable for BinOp {
                     Value::Short(lv_raw) => eval_mul(lv_raw, rv()?),
                     Value::Int(lv_raw) => eval_mul(lv_raw, rv()?),
                     Value::Long(lv_raw) => eval_mul(lv_raw, rv()?),
-                    Value::BigInt(lv_raw) => check_bigint_overflow(eval_mul(lv_raw, rv()?)?),
+                    Value::BigInt(lv_raw) => eval_mul(lv_raw, rv()?),
                     _ => Err(EvalError::UnexpectedValue(format!(
                         "expected BinOp::left to be numeric value, got {0:?}",
                         lv
@@ -235,7 +228,7 @@ impl Evaluable for BinOp {
                     Value::Int(lv_raw) => eval_div(lv_raw, rv()?),
                     Value::Long(lv_raw) => eval_div(lv_raw, rv()?),
                     // MIN / -1  can actually overflow
-                    Value::BigInt(lv_raw) => check_bigint_overflow(eval_div(lv_raw, rv()?)?),
+                    Value::BigInt(lv_raw) => eval_div(lv_raw, rv()?),
                     _ => Err(EvalError::UnexpectedValue(format!(
                         "expected BinOp::left to be numeric value, got {0:?}",
                         lv
@@ -263,7 +256,9 @@ impl Evaluable for BinOp {
                         lv
                     ))),
                 },
-                ArithOp::BitAnd => match lv {
+            },
+            BinOpKind::Bit(op) => match op {
+                BitOp::BitAnd => match lv {
                     Value::Byte(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l & r),
                     Value::Short(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l & r),
                     Value::Int(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l & r),
@@ -274,7 +269,7 @@ impl Evaluable for BinOp {
                         lv
                     ))),
                 },
-                ArithOp::BitOr => match lv {
+                BitOp::BitOr => match lv {
                     Value::Byte(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l | r),
                     Value::Short(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l | r),
                     Value::Int(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l | r),
@@ -285,7 +280,7 @@ impl Evaluable for BinOp {
                         lv
                     ))),
                 },
-                ArithOp::BitXor => match lv {
+                BitOp::BitXor => match lv {
                     Value::Byte(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l ^ r),
                     Value::Short(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l ^ r),
                     Value::Int(lv_raw) => eval_bit_op(lv_raw, rv()?, |l, r| l ^ r),
@@ -301,6 +296,7 @@ impl Evaluable for BinOp {
     }
 }
 
+#[allow(clippy::panic)]
 #[cfg(test)]
 #[allow(clippy::unwrap_used)]
 mod tests {
@@ -310,7 +306,7 @@ mod tests {
     use crate::eval::tests::try_eval_out;
     use ergotree_ir::mir::constant::Constant;
     use ergotree_ir::mir::expr::Expr;
-    use num_bigint::ToBigInt;
+    use num_traits::Bounded;
     use proptest::prelude::*;
     use sigma_test_util::force_any_val;
     use std::rc::Rc;
@@ -385,7 +381,7 @@ mod tests {
     #[test]
     fn bin_or_eval_laziness() {
         let e: Expr = BinOp {
-            kind: BinOpKind::Relation(RelationOp::Or),
+            kind: BinOpKind::Logical(LogicalOp::Or),
             left: Box::new(Expr::Const(true.into())),
             // something that should blow-up the evaluation
             right: Box::new(
@@ -399,13 +395,13 @@ mod tests {
         }
         .into();
         let ctx = Rc::new(force_any_val::<Context>());
-        assert_eq!(eval_out::<bool>(&e, ctx), true);
+        assert!(eval_out::<bool>(&e, ctx));
     }
 
     #[test]
     fn bin_and_eval_laziness() {
         let e: Expr = BinOp {
-            kind: BinOpKind::Relation(RelationOp::And),
+            kind: BinOpKind::Logical(LogicalOp::And),
             left: Box::new(Expr::Const(false.into())),
             // something that should blow-up the evaluation
             right: Box::new(
@@ -419,10 +415,10 @@ mod tests {
         }
         .into();
         let ctx = Rc::new(force_any_val::<Context>());
-        assert_eq!(eval_out::<bool>(&e, ctx), false);
+        assert!(!eval_out::<bool>(&e, ctx));
     }
 
-    fn eval_num_op<T: TryExtractFrom<Value> + Into<Constant>>(
+    fn eval_arith_op<T: TryExtractFrom<Value> + Into<Constant>>(
         op: ArithOp,
         left: T,
         right: T,
@@ -437,6 +433,21 @@ mod tests {
         try_eval_out::<T>(&expr, ctx)
     }
 
+    fn eval_bit_op<T: TryExtractFrom<Value> + Into<Constant>>(
+        op: BitOp,
+        left: T,
+        right: T,
+    ) -> Result<T, EvalError> {
+        let expr: Expr = BinOp {
+            kind: BinOpKind::Bit(op),
+            left: Box::new(left.into().into()),
+            right: Box::new(right.into().into()),
+        }
+        .into();
+        let ctx = Rc::new(force_any_val::<Context>());
+        try_eval_out::<T>(&expr, ctx)
+    }
+
     fn eval_relation_op<T: Into<Constant>>(op: RelationOp, left: T, right: T) -> bool {
         let expr: Expr = BinOp {
             kind: BinOpKind::Relation(op),
@@ -448,38 +459,50 @@ mod tests {
         eval_out::<bool>(&expr, ctx)
     }
 
+    fn eval_logical_op<T: Into<Constant>>(op: LogicalOp, left: T, right: T) -> bool {
+        let expr: Expr = BinOp {
+            kind: BinOpKind::Logical(op),
+            left: Box::new(left.into().into()),
+            right: Box::new(right.into().into()),
+        }
+        .into();
+        let ctx = Rc::new(force_any_val::<Context>());
+        eval_out::<bool>(&expr, ctx)
+    }
+
     #[test]
     fn test_bigint_extremes() {
-        let b = BigInt::from;
+        let b = |n| BigInt256::from(n);
         // Our BigInt should behave like a 256 bit signed (two's complement) integer according to
         // the language spec. These are the max and min values representable:
-        let max = || (b(1) << 255) - 1;
-        let min = || (b(1) << 255) * -1;
+        let max = BigInt256::max_value;
+        let min = BigInt256::min_value;
 
-        // The commented tests below are currently failing due to issue #288.
+        assert!(eval_arith_op(ArithOp::Multiply, max(), b(2)).is_err());
+        assert_eq!(eval_arith_op(ArithOp::Multiply, max(), b(1)), Ok(max()));
+        assert!(eval_arith_op(ArithOp::Multiply, min(), b(2)).is_err());
+        assert_eq!(eval_arith_op(ArithOp::Multiply, min(), b(1)), Ok(min()));
 
-        assert!(eval_num_op(ArithOp::Multiply, max(), b(2)).is_err());
-        assert_eq!(eval_num_op(ArithOp::Multiply, max(), b(1)), Ok(max()));
-        assert!(eval_num_op(ArithOp::Multiply, min(), b(2)).is_err());
-        assert_eq!(eval_num_op(ArithOp::Multiply, min(), b(1)), Ok(min()));
+        assert!(eval_arith_op(ArithOp::Divide, min(), b(-1)).is_err());
+        assert_eq!(
+            eval_arith_op(ArithOp::Divide, min() + b(1), b(-1)),
+            Ok(max())
+        );
+        assert!(eval_arith_op(ArithOp::Divide, b(20), b(0)).is_err());
 
-        assert!(eval_num_op(ArithOp::Divide, min(), b(-1)).is_err());
-        assert_eq!(eval_num_op(ArithOp::Divide, min() + 1, b(-1)), Ok(max()));
-        assert!(eval_num_op(ArithOp::Divide, b(20), b(0)).is_err());
+        assert!(eval_arith_op(ArithOp::Plus, max(), b(1)).is_err());
+        assert_eq!(eval_arith_op(ArithOp::Plus, max(), b(0)), Ok(max()));
+        assert!(eval_arith_op(ArithOp::Plus, min(), b(-1)).is_err());
+        assert_eq!(eval_arith_op(ArithOp::Plus, min(), b(0)), Ok(min()));
 
-        assert!(eval_num_op(ArithOp::Plus, max(), b(1)).is_err());
-        assert_eq!(eval_num_op(ArithOp::Plus, max(), b(0)), Ok(max()));
-        assert!(eval_num_op(ArithOp::Plus, min(), b(-1)).is_err());
-        assert_eq!(eval_num_op(ArithOp::Plus, min(), b(0)), Ok(min()));
+        assert!(eval_arith_op(ArithOp::Minus, max(), b(-1)).is_err());
+        assert_eq!(eval_arith_op(ArithOp::Minus, max(), b(0)), Ok(max()));
+        assert!(eval_arith_op(ArithOp::Minus, min(), b(1)).is_err());
+        assert_eq!(eval_arith_op(ArithOp::Minus, min(), b(0)), Ok(min()));
 
-        assert!(eval_num_op(ArithOp::Minus, max(), b(-1)).is_err());
-        assert_eq!(eval_num_op(ArithOp::Minus, max(), b(0)), Ok(max()));
-        assert!(eval_num_op(ArithOp::Minus, min(), b(1)).is_err());
-        assert_eq!(eval_num_op(ArithOp::Minus, min(), b(0)), Ok(min()));
-
-        assert_eq!(eval_num_op(ArithOp::BitAnd, max(), min()), Ok(b(0)));
-        assert_eq!(eval_num_op(ArithOp::BitOr, max(), min()), Ok(b(-1)));
-        assert_eq!(eval_num_op(ArithOp::BitXor, max(), min()), Ok(b(-1)));
+        assert_eq!(eval_bit_op(BitOp::BitAnd, max(), min()), Ok(b(0)));
+        assert_eq!(eval_bit_op(BitOp::BitOr, max(), min()), Ok(b(-1)));
+        assert_eq!(eval_bit_op(BitOp::BitXor, max(), min()), Ok(b(-1)));
     }
 
     proptest! {
@@ -493,16 +516,16 @@ mod tests {
 
         #[test]
         fn test_num_slong(l in any::<i64>(), r in any::<i64>()) {
-            prop_assert_eq!(eval_num_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
-            prop_assert_eq!(eval_num_op::<i64>(ArithOp::Max, l, r).unwrap(), l.max(r));
-            prop_assert_eq!(eval_num_op::<i64>(ArithOp::Min, l, r).unwrap(), l.min(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
+            prop_assert_eq!(eval_arith_op::<i64>(ArithOp::Max, l, r).unwrap(), l.max(r));
+            prop_assert_eq!(eval_arith_op::<i64>(ArithOp::Min, l, r).unwrap(), l.min(r));
 
-            prop_assert_eq!(eval_num_op(ArithOp::BitAnd, l, r), Ok(l & r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitOr, l, r), Ok(l | r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitXor, l, r), Ok(l ^ r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitAnd, l, r), Ok(l & r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitOr, l, r), Ok(l | r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitXor, l, r), Ok(l ^ r));
 
             prop_assert_eq!(eval_relation_op(RelationOp::Gt, l, r), l > r);
             prop_assert_eq!(eval_relation_op(RelationOp::Lt, l, r), l < r);
@@ -512,16 +535,16 @@ mod tests {
 
         #[test]
         fn test_num_sint(l in any::<i32>(), r in any::<i32>()) {
-            prop_assert_eq!(eval_num_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
-            prop_assert_eq!(eval_num_op::<i32>(ArithOp::Max, l, r).unwrap(), l.max(r));
-            prop_assert_eq!(eval_num_op::<i32>(ArithOp::Min, l, r).unwrap(), l.min(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
+            prop_assert_eq!(eval_arith_op::<i32>(ArithOp::Max, l, r).unwrap(), l.max(r));
+            prop_assert_eq!(eval_arith_op::<i32>(ArithOp::Min, l, r).unwrap(), l.min(r));
 
-            prop_assert_eq!(eval_num_op(ArithOp::BitAnd, l, r), Ok(l & r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitOr, l, r), Ok(l | r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitXor, l, r), Ok(l ^ r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitAnd, l, r), Ok(l & r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitOr, l, r), Ok(l | r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitXor, l, r), Ok(l ^ r));
 
             prop_assert_eq!(eval_relation_op(RelationOp::Gt, l, r), l > r);
             prop_assert_eq!(eval_relation_op(RelationOp::Lt, l, r), l < r);
@@ -531,16 +554,16 @@ mod tests {
 
         #[test]
         fn test_num_sshort(l in any::<i16>(), r in any::<i16>()) {
-            prop_assert_eq!(eval_num_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
-            prop_assert_eq!(eval_num_op::<i16>(ArithOp::Max, l, r).unwrap(), l.max(r));
-            prop_assert_eq!(eval_num_op::<i16>(ArithOp::Min, l, r).unwrap(), l.min(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
+            prop_assert_eq!(eval_arith_op::<i16>(ArithOp::Max, l, r).unwrap(), l.max(r));
+            prop_assert_eq!(eval_arith_op::<i16>(ArithOp::Min, l, r).unwrap(), l.min(r));
 
-            prop_assert_eq!(eval_num_op(ArithOp::BitAnd, l, r), Ok(l & r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitOr, l, r), Ok(l | r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitXor, l, r), Ok(l ^ r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitAnd, l, r), Ok(l & r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitOr, l, r), Ok(l | r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitXor, l, r), Ok(l ^ r));
 
             prop_assert_eq!(eval_relation_op(RelationOp::Gt, l, r), l > r);
             prop_assert_eq!(eval_relation_op(RelationOp::Lt, l, r), l < r);
@@ -550,16 +573,16 @@ mod tests {
 
         #[test]
         fn test_num_sbyte(l in any::<i8>(), r in any::<i8>()) {
-            prop_assert_eq!(eval_num_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
-            prop_assert_eq!(eval_num_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
-            prop_assert_eq!(eval_num_op::<i8>(ArithOp::Max, l, r).unwrap(), l.max(r));
-            prop_assert_eq!(eval_num_op::<i8>(ArithOp::Min, l, r).unwrap(), l.min(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Plus, l, r).ok(), l.checked_add(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Minus, l, r).ok(), l.checked_sub(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Multiply, l, r).ok(), l.checked_mul(r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Divide, l, r).ok(), l.checked_div(r));
+            prop_assert_eq!(eval_arith_op::<i8>(ArithOp::Max, l, r).unwrap(), l.max(r));
+            prop_assert_eq!(eval_arith_op::<i8>(ArithOp::Min, l, r).unwrap(), l.min(r));
 
-            prop_assert_eq!(eval_num_op(ArithOp::BitAnd, l, r), Ok(l & r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitOr, l, r), Ok(l | r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitXor, l, r), Ok(l ^ r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitAnd, l, r), Ok(l & r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitOr, l, r), Ok(l | r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitXor, l, r), Ok(l ^ r));
 
             prop_assert_eq!(eval_relation_op(RelationOp::Gt, l, r), l > r);
             prop_assert_eq!(eval_relation_op(RelationOp::Lt, l, r), l < r);
@@ -569,20 +592,20 @@ mod tests {
 
         #[test]
         fn test_num_bigint(l_long in any::<i64>(), r_long in any::<i64>()) {
-            let l = l_long.to_bigint().unwrap();
-            let r = r_long.to_bigint().unwrap();
-            prop_assert_eq!(eval_num_op(ArithOp::Plus, l.clone(), r.clone()).ok(), l.checked_add(&r));
-            prop_assert_eq!(eval_num_op(ArithOp::Minus, l.clone(), r.clone()).ok(), l.checked_sub(&r));
-            prop_assert_eq!(eval_num_op(ArithOp::Multiply, l.clone(), r.clone()).ok(), l.checked_mul(&r));
-            prop_assert_eq!(eval_num_op(ArithOp::Divide, l.clone(), r.clone()).ok(), l.checked_div(&r));
-            prop_assert_eq!(eval_num_op::<BigInt>(ArithOp::Max, l.clone(),
+            let l = BigInt256::from(l_long);
+            let r = BigInt256::from(r_long);
+            prop_assert_eq!(eval_arith_op(ArithOp::Plus, l.clone(), r.clone()).ok(), l.checked_add(&r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Minus, l.clone(), r.clone()).ok(), l.checked_sub(&r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Multiply, l.clone(), r.clone()).ok(), l.checked_mul(&r));
+            prop_assert_eq!(eval_arith_op(ArithOp::Divide, l.clone(), r.clone()).ok(), l.checked_div(&r));
+            prop_assert_eq!(eval_arith_op::<BigInt256>(ArithOp::Max, l.clone(),
                     r.clone()).unwrap(), l.clone().max(r.clone()));
-            prop_assert_eq!(eval_num_op::<BigInt>(ArithOp::Min, l.clone(),
+            prop_assert_eq!(eval_arith_op::<BigInt256>(ArithOp::Min, l.clone(),
                     r.clone()).unwrap(), l.clone().min(r.clone()));
 
-            prop_assert_eq!(eval_num_op(ArithOp::BitAnd, l.clone(), r.clone()), Ok(&l & &r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitOr, l.clone(), r.clone()), Ok(&l | &r));
-            prop_assert_eq!(eval_num_op(ArithOp::BitXor, l.clone(), r.clone()), Ok(&l ^ &r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitAnd, l.clone(), r.clone()), Ok(&l & &r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitOr, l.clone(), r.clone()), Ok(&l | &r));
+            prop_assert_eq!(eval_bit_op(BitOp::BitXor, l.clone(), r.clone()), Ok(&l ^ &r));
 
             prop_assert_eq!(eval_relation_op(RelationOp::Gt, l.clone(), r.clone()), l > r);
             prop_assert_eq!(eval_relation_op(RelationOp::Lt, l.clone(), r.clone()), l < r);
@@ -591,10 +614,10 @@ mod tests {
         }
 
         #[test]
-        fn test_and_or(l in any::<bool>(), r in any::<bool>()) {
-            prop_assert_eq!(eval_relation_op(RelationOp::And, l, r), l && r);
-            prop_assert_eq!(eval_relation_op(RelationOp::Or, l, r), l || r);
+        fn test_and_or_xor(l in any::<bool>(), r in any::<bool>()) {
+            prop_assert_eq!(eval_logical_op(LogicalOp::And, l, r), l && r);
+            prop_assert_eq!(eval_logical_op(LogicalOp::Or, l, r), l || r);
+            prop_assert_eq!(eval_logical_op(LogicalOp::Xor, l, r), l ^ r);
         }
-
     }
 }
