@@ -10,7 +10,7 @@ impl Evaluable for ExtractScriptBytes {
     fn eval(&self, env: &Env, ctx: &mut EvalContext) -> Result<Value, EvalError> {
         let input_v = self.input.eval(env, ctx)?;
         match input_v {
-            Value::CBox(b) => Ok(ctx.ctx.box_arena.get(&b)?.script_bytes().into()),
+            Value::CBox(b) => Ok(b.script_bytes()?.into()),
             _ => Err(EvalError::UnexpectedValue(format!(
                 "Expected ExtractScriptBytes input to be Value::CBox, got {0:?}",
                 input_v
@@ -39,7 +39,7 @@ mod tests {
         let ctx = Rc::new(force_any_val::<Context>());
         assert_eq!(
             eval_out::<Vec<i8>>(&e, ctx.clone()),
-            ctx.self_box.get_box(&ctx.box_arena).unwrap().script_bytes()
+            ctx.self_box.script_bytes().unwrap()
         );
     }
 }
