@@ -1,11 +1,12 @@
 use crate::eval::Env;
 use ergotree_ir::mir::global_vars::GlobalVars;
 use ergotree_ir::mir::value::Value;
+use ergotree_ir::serialization::SigmaSerializable;
+use ergotree_ir::sigma_protocol::dlog_group;
 
 use super::EvalContext;
 use super::EvalError;
 use super::Evaluable;
-use ergotree_ir::serialization::SigmaSerializable;
 
 impl Evaluable for GlobalVars {
     fn eval(&self, _env: &Env, ectx: &mut EvalContext) -> Result<Value, EvalError> {
@@ -15,8 +16,9 @@ impl Evaluable for GlobalVars {
             GlobalVars::Outputs => Ok(ectx.ctx.outputs.clone().into()),
             GlobalVars::Inputs => Ok(ectx.ctx.inputs.clone().into()),
             GlobalVars::MinerPubKey => {
-                Ok(ectx.ctx.pre_header.miner_pk.sigma_serialize_bytes().into())
+                Ok(ectx.ctx.pre_header.miner_pk.sigma_serialize_bytes()?.into())
             }
+            GlobalVars::GroupGenerator => Ok(dlog_group::generator().into()),
         }
     }
 }
@@ -30,7 +32,8 @@ mod tests {
     use crate::eval::tests::eval_out;
     use ergoscript_compiler::compiler::compile_expr;
     use ergoscript_compiler::script_env::ScriptEnv;
-    use ergotree_ir::ir_ergo_box::IrBoxId;
+    use ergotree_ir::chain::ergo_box::ErgoBox;
+    use ergotree_ir::sigma_protocol::dlog_group::EcPoint;
     use sigma_test_util::force_any_val;
 
     use super::*;
@@ -46,8 +49,8 @@ mod tests {
     fn eval_self_box() {
         let ctx = Rc::new(force_any_val::<Context>());
         assert_eq!(
-            eval_out::<IrBoxId>(&GlobalVars::SelfBox.into(), ctx.clone()),
-            ctx.self_box
+            eval_out::<Rc<ErgoBox>>(&GlobalVars::SelfBox.into(), ctx.clone()).as_ref(),
+            ctx.self_box.as_ref()
         );
     }
 
@@ -55,7 +58,7 @@ mod tests {
     fn eval_outputs() {
         let ctx = Rc::new(force_any_val::<Context>());
         assert_eq!(
-            eval_out::<Vec<IrBoxId>>(&GlobalVars::Outputs.into(), ctx.clone()),
+            eval_out::<Vec<Rc<ErgoBox>>>(&GlobalVars::Outputs.into(), ctx.clone()),
             ctx.outputs
         );
     }
@@ -64,8 +67,17 @@ mod tests {
     fn eval_inputs() {
         let ctx = Rc::new(force_any_val::<Context>());
         assert_eq!(
-            eval_out::<Vec<IrBoxId>>(&GlobalVars::Inputs.into(), ctx.clone()),
+            eval_out::<Vec<Rc<ErgoBox>>>(&GlobalVars::Inputs.into(), ctx.clone()),
             ctx.inputs
         );
     }
+
+    #[test]
+    fn eval_group_generator() {
+        let ctx = Rc::new(force_any_val::<Context>());
+        assert_eq!(
+            eval_out::<EcPoint>(&GlobalVars::GroupGenerator.into(), ctx),
+            dlog_group::generator()
+        );
+    }
 }
