@@ -10,7 +10,7 @@ use super::Evaluable;
 impl Evaluable for PropertyCall {
     fn eval(&self, env: &Env, ectx: &mut EvalContext) -> Result<Value, EvalError> {
         let ov = self.obj.eval(env, ectx)?;
-        smethod_eval_fn(&self.method)(env, ectx, ov, vec![])
+        smethod_eval_fn(&self.method)?(env, ectx, ov, vec![])
     }
 }
 
@@ -20,7 +20,7 @@ mod tests {
     use super::*;
     use crate::eval::context::Context;
     use crate::eval::tests::eval_out;
-    use ergotree_ir::ir_ergo_box::IrBoxId;
+    use ergotree_ir::chain::ergo_box::ErgoBox;
     use ergotree_ir::mir::expr::Expr;
     use ergotree_ir::types::scontext;
     use sigma_test_util::force_any_val;
@@ -32,6 +32,9 @@ mod tests {
             .unwrap()
             .into();
         let ctx = Rc::new(force_any_val::<Context>());
-        assert_eq!(eval_out::<Vec<IrBoxId>>(&pc, ctx.clone()), ctx.data_inputs);
+        assert_eq!(
+            eval_out::<Vec<Rc<ErgoBox>>>(&pc, ctx.clone()),
+            ctx.data_inputs
+        );
     }
 }
