@@ -8,6 +8,7 @@ pub mod verifier;
 
 mod challenge;
 mod crypto_utils;
+mod dht_protocol;
 mod dlog_protocol;
 mod fiat_shamir;
 mod proof_tree;
@@ -20,10 +21,11 @@ use k256::Scalar;
 
 use dlog_protocol::FirstDlogProverMessage;
 use std::convert::TryInto;
-use unchecked_tree::{UncheckedSigmaTree, UncheckedTree};
+use unchecked_tree::UncheckedTree;
 use unproven_tree::{UnprovenLeaf, UnprovenSchnorr};
 
 use self::challenge::Challenge;
+use self::dht_protocol::FirstDhTupleProverMessage;
 use self::unchecked_tree::UncheckedSchnorr;
 
 use derive_more::From;
@@ -41,14 +43,14 @@ pub enum FirstProverMessage {
     /// Discrete log
     FirstDlogProverMessage(FirstDlogProverMessage),
     /// DH tupl
-    FirstDhtProverMessage,
+    FirstDhtProverMessage(FirstDhTupleProverMessage),
 }
 
 impl ProverMessage for FirstProverMessage {
     fn bytes(&self) -> Vec<u8> {
         match self {
             FirstProverMessage::FirstDlogProverMessage(fdpm) => fdpm.bytes(),
-            FirstProverMessage::FirstDhtProverMessage => todo!(),
+            FirstProverMessage::FirstDhtProverMessage(fdhtpm) => fdhtpm.bytes(),
         }
     }
 }
@@ -65,7 +67,9 @@ pub(crate) struct GroupSizedBytes(pub(crate) Box<[u8; GROUP_SIZE]>);
 impl From<GroupSizedBytes> for Scalar {
     fn from(b: GroupSizedBytes) -> Self {
         let sl: &[u8] = b.0.as_ref();
-        Scalar::from_bytes_reduced(sl.try_into().expect(""))
+        #[allow(clippy::unwrap_used)]
+        // since it's 32 bytes it's safe to unwrap
+        Scalar::from_bytes_reduced(sl.try_into().unwrap())
     }
 }
 
