@@ -5,15 +5,16 @@ mod prover_result;
 
 pub mod hint;
 
+use crate::sigma_protocol::dht_protocol;
 use crate::sigma_protocol::fiat_shamir::fiat_shamir_hash_fn;
 use crate::sigma_protocol::fiat_shamir::fiat_shamir_tree_to_bytes;
 use crate::sigma_protocol::proof_tree::ProofTree;
-use crate::sigma_protocol::unchecked_tree::UncheckedLeaf;
+use crate::sigma_protocol::unchecked_tree::UncheckedDhTuple;
 use crate::sigma_protocol::unproven_tree::CandUnproven;
 use crate::sigma_protocol::unproven_tree::CorUnproven;
 use crate::sigma_protocol::unproven_tree::NodePosition;
+use crate::sigma_protocol::unproven_tree::UnprovenDhTuple;
 use crate::sigma_protocol::Challenge;
-use crate::sigma_protocol::UncheckedSigmaTree;
 use crate::sigma_protocol::UnprovenLeaf;
 use ergotree_ir::sigma_protocol::sigma_boolean::SigmaBoolean;
 use ergotree_ir::sigma_protocol::sigma_boolean::SigmaConjectureItems;
@@ -22,7 +23,7 @@ use std::rc::Rc;
 
 pub use context_extension::*;
 use ergotree_ir::ergo_tree::ErgoTree;
-use ergotree_ir::ergo_tree::ErgoTreeParsingError;
+use ergotree_ir::ergo_tree::ErgoTreeError;
 use ergotree_ir::sigma_protocol::sigma_boolean::SigmaConjecture;
 use ergotree_ir::sigma_protocol::sigma_boolean::SigmaProofOfKnowledgeTree;
 pub use prover_result::*;
@@ -41,20 +42,21 @@ use super::unchecked_tree::UncheckedTree;
 use super::unproven_tree::UnprovenConjecture;
 use super::unproven_tree::UnprovenSchnorr;
 use super::unproven_tree::UnprovenTree;
+use super::FirstProverMessage::FirstDhtProverMessage;
+use super::FirstProverMessage::FirstDlogProverMessage;
 
 use crate::eval::context::Context;
 use crate::eval::env::Env;
 use crate::eval::{EvalError, Evaluator};
 
-use derive_more::From;
 use thiserror::Error;
 
 /// Prover errors
-#[derive(Error, PartialEq, Eq, Debug, Clone, From)]
+#[derive(Error, PartialEq, Eq, Debug, Clone)]
 pub enum ProverError {
     /// Failed to parse ErgoTree
     #[error("Ergo tree error: {0}")]
-    ErgoTreeError(ErgoTreeParsingError),
+    ErgoTreeError(ErgoTreeError),
     /// Failed to evaluate ErgoTree
     #[error("Evaluation error: {0}")]
     EvalError(EvalError),
@@ -79,6 +81,21 @@ pub enum ProverError {
     /// Error while tree serialization for Fiat-Shamir hash
     #[error("Fiat-Shamir tree serialization error: {0}")]
     FiatShamirTreeSerializationError(FiatShamirTreeSerializationError),
+    /// Not yet implemented
+    #[error("not yet implemented: {0}")]
+    NotYetImplemented(String),
+}
+
+impl From<ErgoTreeError> for ProverError {
+    fn from(e: ErgoTreeError) -> Self {
+        ProverError::ErgoTreeError(e)
+    }
+}
+
+impl From<FiatShamirTreeSerializationError> for ProverError {
+    fn from(e: FiatShamirTreeSerializationError) -> Self {
+        ProverError::FiatShamirTreeSerializationError(e)
+    }
 }
 
 /// Prover
@@ -86,13 +103,11 @@ pub trait Prover: Evaluator {
     /// Secrets of the prover
     fn secrets(&self) -> &[PrivateInput];
 
-    /**
-     * The comments in this section are taken from the algorithm for the
-     * Sigma-protocol prover as described in the ErgoScript white-paper
-     * https://ergoplatform.org/docs/ErgoScript.pdf , Appendix A
-     *
-     * Generate proofs for the given message for ErgoTree reduced to Sigma boolean expression
-     */
+    /// The comments in this section are taken from the algorithm for the
+    /// Sigma-protocol prover as described in the ErgoScript white-paper
+    /// <https://ergoplatform.org/docs/ErgoScript.pdf>, Appendix A
+    ///  
+    /// Generate proofs for the given message for ErgoTree reduced to Sigma boolean expression
     fn prove(
         &self,
         tree: &ErgoTree,
@@ -102,20 +117,24 @@ pub trait Prover: Evaluator {
         hints_bag: &HintsBag,
     ) -> Result<ProverResult, ProverError> {
         let expr = tree.proposition()?;
-        let proof = self
+        let unchecked_tree_opt = self
             .reduce_to_crypto(expr.as_ref(), env, ctx)
             .map_err(ProverError::EvalError)
             .and_then(|v| match v.sigma_prop {
-                SigmaBoolean::TrivialProp(true) => Ok(UncheckedTree::NoProof),
+                SigmaBoolean::TrivialProp(true) => Ok(None),
                 SigmaBoolean::TrivialProp(false) => Err(ProverError::ReducedToFalse),
                 sb => {
-                    let tree = convert_to_unproven(sb);
+                    let tree = convert_to_unproven(sb)?;
                     let unchecked_tree = prove_to_unchecked(self, tree, message, hints_bag)?;
-                    Ok(UncheckedTree::UncheckedSigmaTree(unchecked_tree))
+                    Ok(Some(unchecked_tree))
                 }
-            });
-        proof.map(|v| ProverResult {
-            proof: serialize_sig(v),
+            })?;
+        let proof = match unchecked_tree_opt {
+            Some(tree) => serialize_sig(tree),
+            None => ProofBytes::Empty,
+        };
+        Ok(ProverResult {
+            proof,
             extension: ContextExtension::empty(),
         })
     }
@@ -123,7 +142,7 @@ pub trait Prover: Evaluator {
 
 /// The comments in this section are taken from the algorithm for the
 /// Sigma-protocol prover as described in the white paper
-/// https://ergoplatform.org/docs/ErgoScript.pdf (Appendix A)
+/// <https://ergoplatform.org/docs/ErgoScript.pdf> (Appendix A)
 // if we are concerned about timing attacks against the prover, we should make sure that this code
 //  takes the same amount of time regardless of which nodes are real and which nodes are simulated
 //  In particular, we should avoid the use of exists and forall, because they short-circuit the evaluation
@@ -135,10 +154,10 @@ fn prove_to_unchecked<P: Prover + ?Sized>(
     unproven_tree: UnprovenTree,
     message: &[u8],
     hints_bag: &HintsBag,
-) -> Result<UncheckedSigmaTree, ProverError> {
+) -> Result<UncheckedTree, ProverError> {
     // Prover Step 1: Mark as real everything the prover can prove
     let step1 = mark_real(prover, unproven_tree, hints_bag)?;
-    dbg!(&step1);
+    // dbg!(&step1);
 
     // Prover Step 2: If the root of the tree is marked "simulated" then the prover does not have enough witnesses
     // to perform the proof. Abort.
@@ -150,12 +169,12 @@ fn prove_to_unchecked<P: Prover + ?Sized>(
     // has the right number of simulated children.
 
     let step3 = polish_simulated(prover, step1)?;
-    dbg!(&step3);
+    // dbg!(&step3);
 
     // Prover Steps 4, 5, and 6 together: find challenges for simulated nodes; simulate simulated leaves;
     // compute commitments for real leaves
     let step6 = simulate_and_commit(step3, hints_bag)?;
-    dbg!(&step6);
+    // dbg!(&step6);
 
     // Prover Steps 7: convert the relevant information in the tree (namely, tree structure, node types,
     // the statements being proven and commitments at the leaves)
@@ -168,11 +187,11 @@ fn prove_to_unchecked<P: Prover + ?Sized>(
     s.append(&mut message.to_vec());
     let root_challenge: Challenge = fiat_shamir_hash_fn(s.as_slice()).into();
     let step8 = step6.with_challenge(root_challenge);
-    dbg!(&step8);
+    // dbg!(&step8);
 
     // Prover Step 9: complete the proof by computing challenges at real nodes and additionally responses at real leaves
     let step9 = proving(prover, step8.into(), hints_bag)?;
-    dbg!(&step9);
+    // dbg!(&step9);
     // Prover Step 10: output the right information into the proof
     convert_to_unchecked(step9)
 }
@@ -192,28 +211,17 @@ fn mark_real<P: Prover + ?Sized>(
     proof_tree::rewrite(unproven_tree.into(), &|tree| {
         Ok(match tree {
             ProofTree::UnprovenTree(unp) => match unp {
-                UnprovenTree::UnprovenLeaf(unp_leaf) => match unp_leaf {
-                    UnprovenLeaf::UnprovenSchnorr(us) => {
-                        // If the node is a leaf, mark it "real'' if either the witness for it is
-                        // available or a hint shows the secret is known to an external participant in multi-signing;
-                        // else mark it "simulated"
-                        let secret_known =
-                            hints_bag.real_images().contains(&unp_leaf.proposition())
-                                || prover.secrets().iter().any(|s| match s {
-                                    PrivateInput::DlogProverInput(dl) => {
-                                        dl.public_image() == us.proposition
-                                    }
-                                    _ => false,
-                                });
-                        Some(
-                            UnprovenSchnorr {
-                                simulated: !secret_known,
-                                ..us.clone()
-                            }
-                            .into(),
-                        )
-                    }
-                },
+                UnprovenTree::UnprovenLeaf(unp_leaf) => {
+                    // If the node is a leaf, mark it "real'' if either the witness for it is
+                    // available or a hint shows the secret is known to an external participant in multi-signing;
+                    // else mark it "simulated"
+                    let secret_known = hints_bag.real_images().contains(&unp_leaf.proposition())
+                        || prover
+                            .secrets()
+                            .iter()
+                            .any(|s| s.public_image() == unp_leaf.proposition());
+                    Some(unp_leaf.clone().with_simulated(!secret_known).into())
+                }
                 UnprovenTree::UnprovenConjecture(unp_conj) => match unp_conj {
                     UnprovenConjecture::CandUnproven(cand) => {
                         // If the node is AND, mark it "real" if all of its children are marked real; else mark it "simulated"
@@ -251,15 +259,22 @@ fn mark_real<P: Prover + ?Sized>(
 }
 
 /// Set positions for children of a unproven inner node (conjecture, so AND/OR/THRESHOLD)
-fn set_positions(uc: UnprovenConjecture) -> UnprovenConjecture {
+fn set_positions(uc: UnprovenConjecture) -> Result<UnprovenConjecture, ProverError> {
     let upd_children = uc
         .children()
+        .try_mapped(|c| match c {
+            ProofTree::UncheckedTree(unch) => Err(ProverError::Unexpected(format!(
+                "set_positions: unexpected UncheckedTree: {:?}",
+                unch
+            ))),
+            ProofTree::UnprovenTree(unp) => Ok(unp),
+        })?
         .enumerated()
-        .mapped(|(idx, utree)| utree.with_position(uc.position().child(idx)));
-    match uc {
+        .mapped(|(idx, utree)| utree.with_position(uc.position().child(idx)).into());
+    Ok(match uc {
         UnprovenConjecture::CandUnproven(cand) => cand.with_children(upd_children).into(),
         UnprovenConjecture::CorUnproven(cor) => cor.with_children(upd_children).into(),
-    }
+    })
 }
 
 /// If the node is OR marked "real",  mark all but one of its children "simulated"
@@ -326,7 +341,7 @@ fn polish_simulated<P: Prover + ?Sized>(
                     } else {
                         cand.clone()
                     };
-                    Ok(Some(set_positions(a.into()).into()))
+                    Ok(Some(set_positions(a.into())?.into()))
                 }
                 UnprovenConjecture::CorUnproven(cor) => {
                     // If the node is marked "simulated", mark all of its children "simulated"
@@ -340,7 +355,7 @@ fn polish_simulated<P: Prover + ?Sized>(
                         // If the node is OR marked "real",  mark all but one of its children "simulated"
                         make_cor_children_simulated(cor.clone())?
                     };
-                    Ok(Some(set_positions(o.into()).into()))
+                    Ok(Some(set_positions(o.into())?.into()))
                 }
             },
         },
@@ -485,19 +500,19 @@ fn simulate_and_commit(
                 let res: ProofTree = match hints_bag
                     .commitments()
                     .into_iter()
-                    .find(|c| c.position() == us.position)
+                    .find(|c| c.position() == &us.position)
                 {
                     Some(cmt_hint) => {
-                        let pt: ProofTree = UnprovenSchnorr {
-                            commitment_opt: Some(
-                                cmt_hint
-                                    .commitment()
-                                    .try_into()
-                                    .map_err(|e: &str| ProverError::Unexpected(e.to_string()))?,
-                            ),
-                            ..us.clone()
-                        }
-                        .into();
+                        let pt: ProofTree =
+                            UnprovenSchnorr {
+                                commitment_opt: Some(
+                                    cmt_hint.commitment().clone().try_into().map_err(
+                                        |e: &str| ProverError::Unexpected(e.to_string()),
+                                    )?,
+                                ),
+                                ..us.clone()
+                            }
+                            .into();
                         pt
                     }
                     None => {
@@ -537,6 +552,62 @@ fn simulate_and_commit(
                 };
                 Ok(Some(res))
             }
+            ProofTree::UnprovenTree(UnprovenTree::UnprovenLeaf(UnprovenLeaf::UnprovenDhTuple(
+                dhu,
+            ))) => {
+                //Steps 5 & 6: pull out commitment from the hints bag, otherwise, compute the commitment(if the node is real),
+                // or simulate it (if the node is simulated)
+
+                // Step 6 (real leaf -- compute the commitment a or take it from the hints bag)
+                let res: Result<ProofTree, _> = hints_bag
+                    .commitments()
+                    .iter()
+                    .find(|c| c.position() == &dhu.position)
+                    .map(|cmt_hint| {
+                        Ok(dhu
+                            .clone()
+                            .with_commitment(match cmt_hint.commitment() {
+                                FirstDlogProverMessage(_) => {
+                                    return Err(ProverError::Unexpected(
+                                        "Step 5 & 6 for UnprovenDhTuple: FirstDlogProverMessage is not expected here".to_string(),
+                                    ))
+                                }
+                                FirstDhtProverMessage(dhtm) => dhtm.clone(),
+                            })
+                            .into())
+                    })
+                    .unwrap_or_else(|| {
+                        if dhu.simulated {
+                            // Step 5 (simulated leaf -- complete the simulation)
+                            if let Some(dhu_challenge) = dhu.challenge_opt.clone() {
+                                let (fm, sm) = dht_protocol::interactive_prover::simulate(
+                                    &dhu.proposition,
+                                    &dhu_challenge,
+                                );
+                                Ok(UncheckedDhTuple {
+                                    proposition: dhu.proposition.clone(),
+                                    commitment_opt: Some(fm),
+                                    challenge: dhu_challenge,
+                                    second_message: sm,
+                                }
+                                .into())
+                            } else {
+                                Err(ProverError::SimulatedLeafWithoutChallenge)
+                            }
+                        } else {
+                            // Step 6 -- compute the commitment
+                            let (r, fm) =
+                                dht_protocol::interactive_prover::first_message(&dhu.proposition);
+                            Ok(UnprovenDhTuple {
+                                commitment_opt: Some(fm),
+                                randomness_opt: Some(r),
+                                ..dhu.clone()
+                            }
+                            .into())
+                        }
+                    });
+                Ok(Some(res?))
+            }
             ProofTree::UncheckedTree(_) => Ok(None),
         }
     })?
@@ -557,15 +628,11 @@ fn proving<P: Prover + ?Sized>(
     proof_tree::rewrite(proof_tree, &|tree| {
         match &tree {
             ProofTree::UncheckedTree(unch) => match unch {
-                UncheckedTree::NoProof => Err(ProverError::Unexpected(
-                    "Unexpected NoProof in proving()".to_string(),
-                )),
-                UncheckedTree::UncheckedSigmaTree(ust) => match ust {
-                    UncheckedSigmaTree::UncheckedLeaf(_) => Ok(None),
-                    UncheckedSigmaTree::UncheckedConjecture(_) => Err(ProverError::Unexpected(
-                        format!("proving: unexpected {:?}", tree),
-                    )),
-                },
+                UncheckedTree::UncheckedLeaf(_) => Ok(None),
+                UncheckedTree::UncheckedConjecture(_) => Err(ProverError::Unexpected(format!(
+                    "proving: unexpected {:?}",
+                    tree
+                ))),
             },
             ProofTree::UnprovenTree(unproven_tree) => match unproven_tree {
                 UnprovenTree::UnprovenConjecture(conj) => match conj {
@@ -638,7 +705,7 @@ fn proving<P: Prover + ?Sized>(
                                 .find(|prover_input| prover_input.public_image() == us.proposition)
                             {
                                 let z = dlog_protocol::interactive_prover::second_message(
-                                    &priv_key,
+                                    priv_key,
                                     us.randomness_opt.ok_or_else(|| {
                                         ProverError::Unexpected(format!(
                                             "empty randomness in {:?}",
@@ -663,6 +730,64 @@ fn proving<P: Prover + ?Sized>(
                             Err(ProverError::RealUnprovenTreeWithoutChallenge)
                         }
                     }
+                    UnprovenLeaf::UnprovenDhTuple(dhu) => {
+                        // If the node is a leaf marked "real", compute its response according to the second prover step
+                        // of the Sigma-protocol given the commitment, challenge, and witness, or pull response from
+                        // the hints bag
+                        if let Some(dhu_challenge) = dhu.challenge_opt.clone() {
+                            let priv_key_opt = prover
+                                .secrets()
+                                .iter()
+                                .find(|s| s.public_image() == dhu.proposition.clone().into());
+                            let z = match priv_key_opt {
+                                Some(PrivateInput::DhTupleProverInput(priv_key)) => match hints_bag
+                                    .own_commitments()
+                                    .iter()
+                                    .find(|c| c.position == dhu.position)
+                                {
+                                    Some(commitment_from_hints_bag) => {
+                                        dht_protocol::interactive_prover::second_message(
+                                            priv_key,
+                                            &commitment_from_hints_bag.secret_randomness,
+                                            &dhu_challenge,
+                                        )
+                                    }
+                                    None => dht_protocol::interactive_prover::second_message(
+                                        priv_key,
+                                        &dhu.randomness_opt.ok_or_else(|| {
+                                            ProverError::Unexpected(format!(
+                                                "empty randomness in {:?}",
+                                                dhu
+                                            ))
+                                        })?,
+                                        &dhu_challenge,
+                                    ),
+                                },
+                                Some(pi) => {
+                                    return Err(ProverError::Unexpected(format!(
+                                        "Expected DH prover input in prover secrets, got {:?}",
+                                        pi
+                                    )))
+                                }
+                                None => {
+                                    return Err(ProverError::NotYetImplemented(
+                                        "when secret not found".to_string(),
+                                    ))
+                                }
+                            };
+                            Ok(Some(
+                                UncheckedDhTuple {
+                                    proposition: dhu.proposition.clone(),
+                                    commitment_opt: None,
+                                    challenge: dhu_challenge,
+                                    second_message: z,
+                                }
+                                .into(),
+                            ))
+                        } else {
+                            Err(ProverError::RealUnprovenTreeWithoutChallenge)
+                        }
+                    }
                 },
                 UnprovenTree::UnprovenLeaf(unp_leaf) => {
                     // if the simulated node is proven by someone else, take it from hints bag
@@ -679,10 +804,18 @@ fn proving<P: Prover + ?Sized>(
     })
 }
 
-fn convert_to_unproven(sb: SigmaBoolean) -> UnprovenTree {
-    match sb {
+fn convert_to_unproven(sb: SigmaBoolean) -> Result<UnprovenTree, ProverError> {
+    Ok(match sb {
         SigmaBoolean::ProofOfKnowledge(pok) => match pok {
-            SigmaProofOfKnowledgeTree::ProveDhTuple(_) => todo!(),
+            SigmaProofOfKnowledgeTree::ProveDhTuple(pdht) => UnprovenDhTuple {
+                proposition: pdht,
+                commitment_opt: None,
+                randomness_opt: None,
+                challenge_opt: None,
+                simulated: false,
+                position: NodePosition::crypto_tree_prefix(),
+            }
+            .into(),
             SigmaProofOfKnowledgeTree::ProveDlog(prove_dlog) => UnprovenSchnorr {
                 proposition: prove_dlog,
                 commitment_opt: None,
@@ -693,40 +826,49 @@ fn convert_to_unproven(sb: SigmaBoolean) -> UnprovenTree {
             }
             .into(),
         },
-        SigmaBoolean::SigmaConjecture(SigmaConjecture::Cand(cand)) => CandUnproven {
-            proposition: cand.clone(),
-            challenge_opt: None,
-            simulated: false,
-            children: cand.items.mapped(|it| convert_to_unproven(it).into()),
-            position: NodePosition::crypto_tree_prefix(),
+        SigmaBoolean::SigmaConjecture(conj) => match conj {
+            SigmaConjecture::Cand(cand) => CandUnproven {
+                proposition: cand.clone(),
+                challenge_opt: None,
+                simulated: false,
+                children: cand
+                    .items
+                    .try_mapped(|it| convert_to_unproven(it).map(Into::into))?,
+                position: NodePosition::crypto_tree_prefix(),
+            }
+            .into(),
+            SigmaConjecture::Cor(cor) => CorUnproven {
+                proposition: cor.clone(),
+                challenge_opt: None,
+                simulated: false,
+                children: cor
+                    .items
+                    .try_mapped(|it| convert_to_unproven(it).map(Into::into))?,
+                position: NodePosition::crypto_tree_prefix(),
+            }
+            .into(),
+            SigmaConjecture::Cthreshold(_) => {
+                return Err(ProverError::NotYetImplemented(
+                    "Cthreshold is not yet implemented".to_string(),
+                ))
+            }
+        },
+        SigmaBoolean::TrivialProp(_) => {
+            return Err(ProverError::Unexpected(
+                "TrivialProp is not expected here".to_string(),
+            ))
         }
-        .into(),
-        SigmaBoolean::SigmaConjecture(SigmaConjecture::Cor(cor)) => CorUnproven {
-            proposition: cor.clone(),
-            challenge_opt: None,
-            simulated: false,
-            children: cor.items.mapped(|it| convert_to_unproven(it).into()),
-            position: NodePosition::crypto_tree_prefix(),
-        }
-        .into(),
-        SigmaBoolean::TrivialProp(_) => panic!("TrivialProp is not expected here"),
-    }
+    })
 }
 
-fn convert_to_unchecked(tree: ProofTree) -> Result<UncheckedSigmaTree, ProverError> {
+fn convert_to_unchecked(tree: ProofTree) -> Result<UncheckedTree, ProverError> {
     match &tree {
         ProofTree::UncheckedTree(unch_tree) => match unch_tree {
-            UncheckedTree::NoProof => Err(ProverError::Unexpected(
-                "convert_to_unchecked: unexpected NoProof".to_string(),
-            )),
-            UncheckedTree::UncheckedSigmaTree(ust) => match ust {
-                UncheckedSigmaTree::UncheckedLeaf(ul) => match ul {
-                    UncheckedLeaf::UncheckedSchnorr(_) => Ok(ust.clone()),
-                },
-                UncheckedSigmaTree::UncheckedConjecture(_) => Err(ProverError::Unexpected(
-                    format!("convert_to_unchecked: unexpected {:?}", tree),
-                )),
-            },
+            UncheckedTree::UncheckedLeaf(_) => Ok(unch_tree.clone()),
+            UncheckedTree::UncheckedConjecture(_) => Err(ProverError::Unexpected(format!(
+                "convert_to_unchecked: unexpected {:?}",
+                tree
+            ))),
         },
         ProofTree::UnprovenTree(unp_tree) => match unp_tree {
             UnprovenTree::UnprovenLeaf(_) => Err(ProverError::Unexpected(format!(
@@ -770,22 +912,25 @@ impl Prover for TestProver {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::sigma_protocol::private_input::DhTupleProverInput;
     use crate::sigma_protocol::private_input::DlogProverInput;
     use ergotree_ir::mir::constant::Constant;
+    use ergotree_ir::mir::constant::Literal;
     use ergotree_ir::mir::expr::Expr;
     use ergotree_ir::mir::sigma_and::SigmaAnd;
     use ergotree_ir::mir::sigma_or::SigmaOr;
-    use ergotree_ir::mir::value::Value;
     use ergotree_ir::types::stype::SType;
     use sigma_test_util::force_any_val;
+    use std::convert::TryFrom;
     use std::rc::Rc;
 
     #[test]
     fn test_prove_true_prop() {
-        let bool_true_tree = ErgoTree::from(Expr::Const(Constant {
+        let bool_true_tree = ErgoTree::try_from(Expr::Const(Constant {
             tpe: SType::SBoolean,
-            v: Value::Boolean(true),
-        }));
+            v: Literal::Boolean(true),
+        }))
+        .unwrap();
         let message = vec![0u8; 100];
 
         let prover = TestProver { secrets: vec![] };
@@ -802,10 +947,11 @@ mod tests {
 
     #[test]
     fn test_prove_false_prop() {
-        let bool_false_tree = ErgoTree::from(Expr::Const(Constant {
+        let bool_false_tree = ErgoTree::try_from(Expr::Const(Constant {
             tpe: SType::SBoolean,
-            v: Value::Boolean(false),
-        }));
+            v: Literal::Boolean(false),
+        }))
+        .unwrap();
         let message = vec![0u8; 100];
 
         let prover = TestProver { secrets: vec![] };
@@ -824,7 +970,7 @@ mod tests {
     fn test_prove_pk_prop() {
         let secret = DlogProverInput::random();
         let pk = secret.public_image();
-        let tree = ErgoTree::from(Expr::Const(pk.into()));
+        let tree = ErgoTree::try_from(Expr::Const(pk.into())).unwrap();
         let message = vec![0u8; 100];
 
         let prover = TestProver {
@@ -850,7 +996,7 @@ mod tests {
         let expr: Expr = SigmaAnd::new(vec![Expr::Const(pk1.into()), Expr::Const(pk2.into())])
             .unwrap()
             .into();
-        let tree: ErgoTree = expr.into();
+        let tree: ErgoTree = expr.try_into().unwrap();
         let message = vec![0u8; 100];
 
         let prover = TestProver {
@@ -882,7 +1028,7 @@ mod tests {
         ])
         .unwrap()
         .into();
-        let tree: ErgoTree = expr.into();
+        let tree: ErgoTree = expr.try_into().unwrap();
         let message = vec![0u8; 100];
 
         let prover = TestProver {
@@ -907,7 +1053,7 @@ mod tests {
         let expr: Expr = SigmaOr::new(vec![Expr::Const(pk1.into()), Expr::Const(pk2.into())])
             .unwrap()
             .into();
-        let tree: ErgoTree = expr.into();
+        let tree: ErgoTree = expr.try_into().unwrap();
         let message = vec![0u8; 100];
 
         let prover = TestProver {
@@ -939,7 +1085,7 @@ mod tests {
         ])
         .unwrap()
         .into();
-        let tree: ErgoTree = expr.into();
+        let tree: ErgoTree = expr.try_into().unwrap();
         let message = vec![0u8; 100];
 
         let prover = TestProver {
@@ -954,4 +1100,25 @@ mod tests {
         );
         assert_ne!(res.unwrap().proof, ProofBytes::Empty);
     }
+
+    #[test]
+    fn test_prove_dht_prop() {
+        let secret = DhTupleProverInput::random();
+        let pi = secret.public_image();
+        let tree = ErgoTree::try_from(Expr::Const(pi.clone().into())).unwrap();
+        let message = vec![0u8; 100];
+
+        let prover = TestProver {
+            secrets: vec![PrivateInput::DhTupleProverInput(secret)],
+        };
+        let res = prover.prove(
+            &tree,
+            &Env::empty(),
+            Rc::new(force_any_val::<Context>()),
+            message.as_slice(),
+            &HintsBag::empty(),
+        );
+        assert!(res.is_ok());
+        assert_ne!(res.unwrap().proof, ProofBytes::Empty);
+    }
 }
