@@ -1,10 +1,13 @@
 //! Unchecked proof tree types
 
+use ergotree_ir::sigma_protocol::sigma_boolean::ProveDhTuple;
 use ergotree_ir::sigma_protocol::sigma_boolean::ProveDlog;
 use ergotree_ir::sigma_protocol::sigma_boolean::SigmaBoolean;
 use ergotree_ir::sigma_protocol::sigma_boolean::SigmaConjectureItems;
 use ergotree_ir::sigma_protocol::sigma_boolean::SigmaProofOfKnowledgeTree;
 
+use super::dht_protocol::FirstDhTupleProverMessage;
+use super::dht_protocol::SecondDhTupleProverMessage;
 use super::proof_tree::ConjectureType;
 use super::proof_tree::ProofTree;
 use super::proof_tree::ProofTreeConjecture;
@@ -17,70 +20,61 @@ use super::{
 
 use derive_more::From;
 
-/// Unchecked tree
-#[derive(PartialEq, Debug, Clone, From)]
-pub enum UncheckedTree {
-    /// No proof needed
-    NoProof,
-    /// Unchecked sigma tree
-    UncheckedSigmaTree(UncheckedSigmaTree),
-}
-
-impl UncheckedTree {
-    pub(crate) fn as_tree_kind(&self) -> ProofTreeKind {
-        match self {
-            UncheckedTree::NoProof => panic!("NoProof has not ProofTreeKind representation"),
-            UncheckedTree::UncheckedSigmaTree(ust) => ust.as_tree_kind(),
-        }
-    }
-
-    pub(crate) fn challenge(&self) -> Option<Challenge> {
-        match self {
-            UncheckedTree::NoProof => None,
-            UncheckedTree::UncheckedSigmaTree(ust) => Some(ust.challenge()),
-        }
-    }
-}
-
 /// Unchecked sigma tree
 #[derive(PartialEq, Debug, Clone, From)]
-pub enum UncheckedSigmaTree {
+pub enum UncheckedTree {
     /// Unchecked leaf
     UncheckedLeaf(UncheckedLeaf),
     /// Unchecked conjecture (OR, AND, ...)
     UncheckedConjecture(UncheckedConjecture),
 }
 
-impl UncheckedSigmaTree {
+impl UncheckedTree {
     /// Get challenge
     pub(crate) fn challenge(&self) -> Challenge {
         match self {
-            UncheckedSigmaTree::UncheckedLeaf(UncheckedLeaf::UncheckedSchnorr(us)) => {
-                us.challenge.clone()
-            }
-            UncheckedSigmaTree::UncheckedConjecture(uc) => uc.challenge(),
+            UncheckedTree::UncheckedLeaf(ul) => ul.challenge(),
+            UncheckedTree::UncheckedConjecture(uc) => uc.challenge(),
         }
     }
 
     pub(crate) fn as_tree_kind(&self) -> ProofTreeKind {
         match self {
-            UncheckedSigmaTree::UncheckedLeaf(ul) => ProofTreeKind::Leaf(ul),
-            UncheckedSigmaTree::UncheckedConjecture(uc) => ProofTreeKind::Conjecture(uc),
+            UncheckedTree::UncheckedLeaf(ul) => ProofTreeKind::Leaf(ul),
+            UncheckedTree::UncheckedConjecture(uc) => ProofTreeKind::Conjecture(uc),
+        }
+    }
+
+    pub(crate) fn with_challenge(self, challenge: Challenge) -> Self {
+        match self {
+            UncheckedTree::UncheckedLeaf(ul) => ul.with_challenge(challenge).into(),
+            UncheckedTree::UncheckedConjecture(uc) => uc.with_challenge(challenge).into(),
         }
     }
 }
 
-impl From<UncheckedSchnorr> for UncheckedSigmaTree {
-    fn from(v: UncheckedSchnorr) -> Self {
-        UncheckedSigmaTree::UncheckedLeaf(v.into())
-    }
-}
-
 /// Unchecked leaf
-#[derive(PartialEq, Debug, Clone)]
+#[derive(PartialEq, Debug, Clone, From)]
 pub enum UncheckedLeaf {
     /// Unchecked Schnorr
     UncheckedSchnorr(UncheckedSchnorr),
+    UncheckedDhTuple(UncheckedDhTuple),
+}
+
+impl UncheckedLeaf {
+    pub fn challenge(&self) -> Challenge {
+        match self {
+            UncheckedLeaf::UncheckedSchnorr(us) => us.challenge.clone(),
+            UncheckedLeaf::UncheckedDhTuple(udht) => udht.challenge.clone(),
+        }
+    }
+
+    pub fn with_challenge(self, challenge: Challenge) -> Self {
+        match self {
+            UncheckedLeaf::UncheckedSchnorr(us) => us.with_challenge(challenge).into(),
+            UncheckedLeaf::UncheckedDhTuple(udht) => udht.with_challenge(challenge).into(),
+        }
+    }
 }
 
 impl ProofTreeLeaf for UncheckedLeaf {
@@ -89,21 +83,19 @@ impl ProofTreeLeaf for UncheckedLeaf {
             UncheckedLeaf::UncheckedSchnorr(us) => SigmaBoolean::ProofOfKnowledge(
                 SigmaProofOfKnowledgeTree::ProveDlog(us.proposition.clone()),
             ),
+            UncheckedLeaf::UncheckedDhTuple(dhu) => SigmaBoolean::ProofOfKnowledge(
+                SigmaProofOfKnowledgeTree::ProveDhTuple(dhu.proposition.clone()),
+            ),
         }
     }
     fn commitment_opt(&self) -> Option<FirstProverMessage> {
         match self {
             UncheckedLeaf::UncheckedSchnorr(us) => us.commitment_opt.clone().map(Into::into),
+            UncheckedLeaf::UncheckedDhTuple(udh) => udh.commitment_opt.clone().map(Into::into),
         }
     }
 }
 
-impl From<UncheckedSchnorr> for UncheckedLeaf {
-    fn from(us: UncheckedSchnorr) -> Self {
-        UncheckedLeaf::UncheckedSchnorr(us)
-    }
-}
-
 #[derive(PartialEq, Debug, Clone)]
 pub struct UncheckedSchnorr {
     pub proposition: ProveDlog,
@@ -112,9 +104,35 @@ pub struct UncheckedSchnorr {
     pub second_message: SecondDlogProverMessage,
 }
 
+impl UncheckedSchnorr {
+    pub fn with_challenge(self, challenge: Challenge) -> Self {
+        UncheckedSchnorr { challenge, ..self }
+    }
+}
+
 impl From<UncheckedSchnorr> for UncheckedTree {
     fn from(us: UncheckedSchnorr) -> Self {
-        UncheckedTree::UncheckedSigmaTree(us.into())
+        UncheckedTree::UncheckedLeaf(us.into())
+    }
+}
+
+impl From<UncheckedDhTuple> for UncheckedTree {
+    fn from(dh: UncheckedDhTuple) -> Self {
+        UncheckedTree::UncheckedLeaf(dh.into())
+    }
+}
+
+#[derive(PartialEq, Debug, Clone)]
+pub struct UncheckedDhTuple {
+    pub proposition: ProveDhTuple,
+    pub commitment_opt: Option<FirstDhTupleProverMessage>,
+    pub challenge: Challenge,
+    pub second_message: SecondDhTupleProverMessage,
+}
+
+impl UncheckedDhTuple {
+    pub fn with_challenge(self, challenge: Challenge) -> Self {
+        UncheckedDhTuple { challenge, ..self }
     }
 }
 
@@ -122,16 +140,16 @@ impl From<UncheckedSchnorr> for UncheckedTree {
 pub enum UncheckedConjecture {
     CandUnchecked {
         challenge: Challenge,
-        children: SigmaConjectureItems<UncheckedSigmaTree>,
+        children: SigmaConjectureItems<UncheckedTree>,
     },
     CorUnchecked {
         challenge: Challenge,
-        children: SigmaConjectureItems<UncheckedSigmaTree>,
+        children: SigmaConjectureItems<UncheckedTree>,
     },
 }
 
 impl UncheckedConjecture {
-    pub fn with_children(self, new_children: SigmaConjectureItems<UncheckedSigmaTree>) -> Self {
+    pub fn with_children(self, new_children: SigmaConjectureItems<UncheckedTree>) -> Self {
         match self {
             UncheckedConjecture::CandUnchecked {
                 challenge,
@@ -150,7 +168,7 @@ impl UncheckedConjecture {
         }
     }
 
-    pub fn children_ust(self) -> SigmaConjectureItems<UncheckedSigmaTree> {
+    pub fn children_ust(self) -> SigmaConjectureItems<UncheckedTree> {
         match self {
             UncheckedConjecture::CandUnchecked {
                 challenge: _,
@@ -175,6 +193,25 @@ impl UncheckedConjecture {
             } => challenge.clone(),
         }
     }
+
+    pub fn with_challenge(self, challenge: Challenge) -> Self {
+        match self {
+            UncheckedConjecture::CandUnchecked {
+                challenge: _,
+                children,
+            } => UncheckedConjecture::CandUnchecked {
+                challenge,
+                children,
+            },
+            UncheckedConjecture::CorUnchecked {
+                challenge: _,
+                children,
+            } => UncheckedConjecture::CorUnchecked {
+                challenge,
+                children,
+            },
+        }
+    }
 }
 
 impl ProofTreeConjecture for UncheckedConjecture {
